import { Diagnostic } from "@siteimprove/alfa-act";
import { Array } from "@siteimprove/alfa-array";
import type { Device } from "@siteimprove/alfa-device";
import type { Element, Node } from "@siteimprove/alfa-dom";
import type { Equatable } from "@siteimprove/alfa-equatable";
import type { Hash, Hashable } from "@siteimprove/alfa-hash";
import type { Serializable } from "@siteimprove/alfa-json";
import { Map } from "@siteimprove/alfa-map";
import type { Result } from "@siteimprove/alfa-result";
import { Context } from "@siteimprove/alfa-selector";
import { Sequence } from "@siteimprove/alfa-sequence";
import type { Longhands, Shorthands } from "@siteimprove/alfa-style";
import type * as json from "@siteimprove/alfa-json";
import type { Contrast } from "../common/diagnostic/contrast.js";
type Name = Longhands.Name | Shorthands.Name;
/**
 * @public
 */
export declare class ElementDistinguishable implements Equatable, Hashable, Serializable {
    static of(distinguishingProperties?: Iterable<ElementDistinguishable.Property>, style?: Iterable<readonly [Name, string]>, pairings?: Iterable<Contrast.Pairing<["container", "link"]>>): ElementDistinguishable;
    private readonly _distinguishingProperties;
    private readonly _style;
    private readonly _pairings;
    protected constructor(distinguishingProperties: ReadonlyArray<ElementDistinguishable.Property>, style: Map<Name, string>, pairings: ReadonlyArray<Contrast.Pairing<["container", "link"]>>);
    get distinguishingProperties(): ReadonlyArray<ElementDistinguishable.Property>;
    get style(): Map<Name, string>;
    get pairings(): ReadonlyArray<Contrast.Pairing<["container", "link"]>>;
    withDistinguishingProperties(distinguishingProperties: ReadonlyArray<ElementDistinguishable.Property>): ElementDistinguishable;
    withStyle(...styles: ReadonlyArray<readonly [Name, string]>): ElementDistinguishable;
    withPairings(pairings: ReadonlyArray<Contrast.Pairing<["container", "link"]>>): ElementDistinguishable;
    equals(value: ElementDistinguishable): boolean;
    equals(value: unknown): value is this;
    hash(hash: Hash): void;
    toJSON(): ElementDistinguishable.JSON;
}
/**
 * @public
 */
export declare namespace ElementDistinguishable {
    interface JSON {
        [key: string]: json.JSON;
        distinguishingProperties: Array<ElementDistinguishable.Property>;
        style: Map.JSON<Name, string>;
        pairings: Array<Contrast.Pairing.JSON<["container", "link"]>>;
    }
    type Property = Name | "contrast";
    function from(element: Element, device: Device, target: Element, context: Context | undefined, distinguishingProperties: Iterable<ElementDistinguishable.Property>, pairings: Iterable<Contrast.Pairing<["container", "link"]>>): ElementDistinguishable;
}
/**
 * @public
 */
export declare class DistinguishingStyles extends Diagnostic {
    static of(message: string, defaultStyles?: Iterable<Result<ElementDistinguishable>>, hoverStyles?: Iterable<Result<ElementDistinguishable>>, focusStyles?: Iterable<Result<ElementDistinguishable>>): DistinguishingStyles;
    private readonly _defaultStyles;
    private readonly _hoverStyles;
    private readonly _focusStyles;
    protected constructor(message: string, defaultStyles: Sequence<Result<ElementDistinguishable>>, hoverStyles: Sequence<Result<ElementDistinguishable>>, focusStyles: Sequence<Result<ElementDistinguishable>>);
    get defaultStyles(): Iterable<Result<ElementDistinguishable>>;
    get hoverStyles(): Iterable<Result<ElementDistinguishable>>;
    get focusStyles(): Iterable<Result<ElementDistinguishable>>;
    equals(value: DistinguishingStyles): boolean;
    equals(value: unknown): value is this;
    toJSON(options?: Node.SerializationOptions): DistinguishingStyles.JSON;
}
/**
 * @public
 */
export declare namespace DistinguishingStyles {
    interface JSON extends Diagnostic.JSON {
        defaultStyle: Sequence.JSON<Result<ElementDistinguishable>>;
        hoverStyle: Sequence.JSON<Result<ElementDistinguishable>>;
        focusStyle: Sequence.JSON<Result<ElementDistinguishable>>;
    }
    function isDistinguishingStyles(value: Diagnostic): value is DistinguishingStyles;
    function isDistinguishingStyles(value: unknown): value is DistinguishingStyles;
}
export {};
//# sourceMappingURL=diagnostics.d.ts.map